# Jednoduchá substituční šifra
#
# na základě klíče vyrobíme substituční abecedu, při šifrování
# nahrazujeme písmena otevřeného textu odpovídajícími prvky
# substituční abecedy, při dešifrování se úloha obou abeced
# prohodí

abeceda = "abcdefghijklmnopqrstuvwxyz"

# vytvoření substituční abecedy
#  - nejprve vkládáme jedinečná písmena textového klíče
#  - doplňujeme zbylými písmeny v pořadí od následníka
# aktuálně posledního písmena klíče
def subst_abc(klic):
    vysledek = []
    # unikátní písmena z klíče
    for pismeno in klic.lower():
        if pismeno in abeceda and pismeno not in vysledek:
            vysledek.append(pismeno)
    # pořadí následujícího písmena
    index = abeceda.find(vysledek[-1]) + 1
    # doplnění zbylých písmen v abecedním pořadí (s přetečením)
    delka_abc = len(abeceda)
    for i in range(delka_abc):
        pismeno = abeceda[(index + i) % delka_abc]
        if pismeno not in vysledek:
            vysledek.append(pismeno)
    return "".join(vysledek) # vrátíme řetězec

# náhrada prvků z 'co' odpovídajícími prvky z 'cim'
def substituce(text, co, cim):
    vysledek = list(text) # budeme pracovat na seznamu
    for i in range(len(vysledek)):
        index = co.find(vysledek[i]) # je znak v 'co'
        if index >= 0:
            vysledek[i] = cim[index] # náhrada z 'cim'
    return "".join(vysledek) # vrátíme řetězec
    
# šifrování
# znaky normální abecedy měníme za prvky substituční abecedy
def zasifruj(text, klic):
    return substituce(text.lower(), abeceda, subst_abc(klic))

# dešifrování
# znaky substutuční abecedy měníme za prvky normální abecedy
def rozsifruj(text, klic):
    return substituce(text.lower(), subst_abc(klic), abeceda)
    
puvodni = \
"Toto je velmi tajna zprava o stavu programovani "\
"Pythonu na nasem gymnaziu, kterou potrebujeme "\
"dorucit svym spoluzakum."
print(puvodni)

print("-----------------------------------")
klic = "Nejaky tajny klic"
odeslano = zasifruj(puvodni, klic)
print(odeslano)
prijato = rozsifruj(odeslano, klic)
print(prijato)
print("-----------------------------------")
klic = "Zcela jiny sifrovaci klic"
odeslano = zasifruj(puvodni, klic)
print(odeslano)
prijato = rozsifruj(odeslano, klic)
print(prijato)
